/* zorglub

Copyright 2014 Miller Puckette.  BSD license.
*/

#include "m_pd.h"
#include <math.h>
static t_class *zorglub_class;

typedef struct _zorglub
{
    t_object x_obj;
    t_symbol *x_probtabname;
    t_symbol *x_excesstabname;
} t_zorglub;

static void *zorglub_new(t_symbol *probtabname, t_symbol *excesstabname)
{
    t_zorglub *x = (t_zorglub *)pd_new(zorglub_class);
    outlet_new(&x->x_obj, gensym("float"));
    x->x_probtabname = probtabname;
    x->x_excesstabname = excesstabname;
    if (! *excesstabname->s_name)
        pd_error(x, "probability and/or excess table name not given");
    return (x);
}

static void zorglub_bang(t_zorglub *x)
{
    t_garray *probarray, *excessarray;
    t_word *probvec, *excessvec;
    int probn, excessn, n, i, maxindex;
    float sum, invsum, maximum;
    if (!(probarray = (t_garray *)pd_findbyclass(x->x_probtabname,
        garray_class)))
    {
        if (*x->x_probtabname->s_name)
            pd_error(x, "zorglub: %s: no such array",
                x->x_probtabname->s_name);
        return;;
    }
    if (!garray_getfloatwords(probarray, &probn, &probvec))
    {
        pd_error(x, "%s: bad template for zorglub", x->x_probtabname->s_name);
        return;
    }
    if (!(excessarray = (t_garray *)pd_findbyclass(x->x_excesstabname,
        garray_class)))
    {
        if (*x->x_excesstabname->s_name)
            pd_error(x, "zorglub: %s: no such array",
                x->x_excesstabname->s_name);
        return;;
    }
    if (!garray_getfloatwords(excessarray, &excessn, &excessvec))
    {
        pd_error(x, "%s: bad template for zorglub", x->x_excesstabname->s_name);
        return;
    }
    n = (probn < excessn ? probn : excessn);
    for (i = 0, sum = 0; i < n; i++)
        sum += probvec[i].w_float;
    if (sum < 1e-6)
        sum = 1e-6;
   invsum = 1./sum;
    for (i = 0; i < n; i++)
        excessvec[i].w_float += probvec[i].w_float * invsum;
    for (i = 0, sum = 0; i < n; i++)
        sum += probvec[i].w_float;
    for (i = 0, maximum = -1e12, maxindex = 0; i < n; i++)
        if (excessvec[i].w_float > maximum)
            maximum = excessvec[i].w_float, maxindex = i;
    excessvec[maxindex].w_float -= 1.;
    outlet_float(x->x_obj.ob_outlet, (float)maxindex);
    garray_redraw(excessarray);
}

static void zorglub_print(t_zorglub *x)
{
}

void zorglub_setup(void)
{
    zorglub_class = class_new(gensym("zorglub"), (t_newmethod)zorglub_new,
    	0, sizeof(t_zorglub), 0, A_DEFSYMBOL, A_DEFSYMBOL, 0);
    class_addmethod(zorglub_class, (t_method)zorglub_print, gensym("print"), 0);
    class_addbang(zorglub_class, zorglub_bang);
}
